import 'reflect-metadata'
import 'dotenv/config'
/* IF auth:firebase */ import admin from 'firebase-admin'
/* IF gql:true,auth:jwt */ import jwt from 'jsonwebtoken'
/* IF gql:true,express:true */ import { ApolloServer } from 'apollo-server-express'
/* IF gql:true,express:false */ import { ApolloServer } from 'apollo-server'
/* IF gql:true */ import { ApolloServerPluginDrainHttpServer } from 'apollo-server-core'
/* IF gql:true */ import http from 'http'
import app from '../src/server/initApp'
import '../src/server/initDatabaseClient'
/* IF gql:true */ import { buildSchemaSync } from 'type-graphql'
/* IF gql:true */ import * as resolversObj from '../src/resolvers'
/* IF gql:true */ import { objectToArray } from '../src/utils'

// IF auth:firebase
// initialize firebase admin
let firebaseAdmin: admin.app.App
try {
  firebaseAdmin = admin.initializeApp({
    credential: admin.credential.cert(JSON.parse(process.env.FIREBASE_ADMIN_TOKEN))
  })
} catch (e) {
  console.error('Error parsing FIREBASE_ADMIN_TOKEN', e)
}
// ENDIF

// IF gql:true
// schema auth checker
function authChecker(root: { context: { user?: { uid: string } } }) {
  return root.context.user?.uid ? true : false
}

// build schema
const resolvers = objectToArray<Function>(resolversObj) as [Function, ...Function[]]
const schema = buildSchemaSync({ resolvers, authChecker })
// ENDIF

// IF gql:true,auth:jwt
// get auth headers from request context and verify token
const { AUTH_SECRET } = process.env
async function context({ req }: { req: { headers: { authorization?: string } } }) {
  if (req.headers.authorization && req.headers.authorization.indexOf('Bearer ') === 0) {
    try {
      const accessToken = req.headers.authorization.slice(7)
      const user = jwt.verify(accessToken, AUTH_SECRET)
      return { user }
    } catch (e) {
      console.error('Auth error:', e)
    }
  }
}
// ENDIF

// IF gql:true,auth:firebase
// get auth headers from request context and verify token
async function context({ req }: { req: any }) {
  if (req.headers.authorization && req.headers.authorization.indexOf('Bearer ') === 0) {
    try {
      const accessToken = req.headers.authorization.slice(7)
      const user = await firebaseAdmin.auth().verifyIdToken(accessToken)
      return { user }
    } catch (e) {
      console.error('Auth error:', e)
    }
  }
}
// ENDIF

// IF express:true
// create HTTP server
const httpServer = http.createServer(app)
// ENDIF

// IF gql:true,express:true
// init apollo server
const server = new ApolloServer({
  schema,
  context,
  introspection: true,
  plugins: [ApolloServerPluginDrainHttpServer({ httpServer })],
})

// link apollo server with express
server.start().then(() => {
  server.applyMiddleware({ app, path: '/api' })
})
// ENDIF

// IF express:true
export default httpServer
// ENDIF

// IF gql:true,express:false
// TODO: bind to path /api?
const server = new ApolloServer({
  schema,
  context,
  introspection: true,
  path: '/api'
})

export default server
// ENDIF
