#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = __importDefault(require("yargs"));
const inquirer_1 = __importDefault(require("inquirer"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = __importDefault(require("child_process"));
const chalk_1 = __importDefault(require("chalk"));
const questions_1 = __importDefault(require("./questions"));
const utils_1 = require("./utils");
const files_json_1 = __importDefault(require("./files.json"));
const yargsOpts = {
    'customize': {
        alias: 'c'
    }
};
(async () => {
    const { _: [name], customize } = await yargs_1.default.options(yargsOpts).argv;
    const defaults = customize ? {} : (0, utils_1.getAnswers)(questions_1.default, name);
    const { options: o, firebase_admin_token, ...answers } = await inquirer_1.default.prompt(questions_1.default, { ...defaults, name });
    const inverseOptions = Object.fromEntries((questions_1.default.find(q => q.name === 'options') || { choices: [] }).choices.filter(c => o.indexOf(c.value) < 0 && c.type !== 'separator').map(c => [c.value, false]));
    // parse firebase admin token
    if (firebase_admin_token) {
        answers.firebase_admin_token = JSON.stringify(JSON.parse(firebase_admin_token));
    }
    const options = { ...answers, ...inverseOptions, ...Object.fromEntries(o.map((v) => [v, true])) };
    console.log('lang', yargs_1.default.locale());
    // create folder for project
    const projectFolder = path_1.default.resolve(process.cwd(), `./${options.name}`);
    if (fs_1.default.existsSync(projectFolder)) {
        console.error(`The directory ${options.name} already exists`);
        process.exit(1);
    }
    fs_1.default.mkdirSync(projectFolder, { recursive: true });
    // generate package.json and write it to the project folder
    const json = await (0, utils_1.generatePackageJson)(options);
    fs_1.default.writeFileSync(path_1.default.resolve(process.cwd(), `./${options.name}/package.json`), JSON.stringify(json, null, 2));
    // generate dotenv file
    const content = await (0, utils_1.generateEnvironmentVariables)(options);
    fs_1.default.writeFileSync(path_1.default.resolve(process.cwd(), `./${options.name}/.env`), content);
    // run npm install in the project folder
    if (!process.env.NODE_ENV || process.env.NODE_ENV !== 'development') {
        console.log('Installing packages...');
        child_process_1.default.execSync(`cd ./${options.name} && npm install && cd ../`, { stdio: [0, 1, 2] });
    }
    // construct files
    const regex = /(?:\n.*{|.*)\/[\/*] IF ([a-zA-Z0-9:,]+)(?: \*\/ ?)?\}?(.+\n?|[\s\S]+?(?:ENDIF)(?:.+)?)/gm;
    try {
        files_json_1.default.filter(({ rules }) => !rules || (0, utils_1.filterRules)(rules, options)).forEach(({ path: templateFile, replace = {} }) => {
            const templateContent = fs_1.default.readFileSync(path_1.default.resolve(__dirname, './files', templateFile), 'utf8');
            const fileContent = templateContent
                .replaceAll(regex, (match, rules, snippet) => (0, utils_1.filterRules)(rules, options) ? snippet : '')
                .replace(/(.*\{?\/\*\ ENDIF \*\/\}?\n?|.*\/\/\ ENDIF)\n?/g, '')
                .replace(new RegExp(replace ? Object.keys(replace).map(key => key).join('|') : '$^', 'gi'), (match) => match ? options[replace[match]] : '')
                .replace(/[\r\n]\s+[\r\n]/g, '\r\n\r\n');
            const filePath = path_1.default.resolve(process.cwd(), `./${options.name}`, templateFile);
            const fileDirname = path_1.default.dirname(filePath);
            fs_1.default.mkdirSync(fileDirname, { recursive: true });
            fs_1.default.writeFileSync(filePath, fileContent);
        });
    }
    catch (e) {
        console.error(e);
    }
    // generate firebase config file
    if (answers.firebase_config) {
        const filePath = path_1.default.resolve(process.cwd(), `./${options.name}`, 'src/firebase-config.json');
        const fileDirname = path_1.default.dirname(filePath);
        fs_1.default.mkdirSync(fileDirname, { recursive: true });
        const config = new Function("return " + answers.firebase_config)();
        fs_1.default.writeFileSync(filePath, JSON.stringify(config, null, 2));
    }
    // git init
    child_process_1.default.execSync(`git init ./${options.name}`, { stdio: [0, 1, 2] });
    console.log(`${chalk_1.default.green("Success!")} Created ${options.name} at ${projectFolder}.\n
In that directory, you can run several commands:
  * npm start: Starts the development server
  * npm run build: Bundles the app into static files for production.\n
We suggest that you begin by typing:
  cd ./${options.name}
  npm start`);
})();
//# sourceMappingURL=index.js.map